import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { map, tap } from 'rxjs/operators';

import {
  ActionType,
  RequestDiscover,
  RequestDiscoverStop,
  ChangeDiscoveryStatus,
} from './request.action';
import { DiscoveryService } from '../../../service/discovery.service';
import { DiscoveryStatusType } from 'src/app/core/type';

@Injectable()
export class RequestEffects {
  @Effect({ dispatch: false })
  requestDiscover$ = this.actions$.pipe(
    ofType<RequestDiscover>(ActionType.RequestDiscover),
    map((action) => {
      this.discoveryService.discover(action.payload.discoverySession.discoverRequestInfo);
    }),
  );

  @Effect({ dispatch: false })
  requestDiscoverStop$ = this.actions$.pipe(
    ofType<RequestDiscoverStop>(ActionType.RequestDiscoverStop),
    map((action) => action.payload),
    map((payload) => {
      this.discoveryService.discoverStop(payload.discoverySession.discoverRequestInfo.requesterID, payload.discoverySession.requestID);
    })
  );

  constructor(
    private store: Store<any>,
    private actions$: Actions,
    private router: Router,
    private discoveryService: DiscoveryService,
  ) { }
}
