import { ElementRef } from '@angular/core';
import { Action } from '@ngrx/store';
import { Host, Port, Service } from '@overflow/model/discovery';
import { TargetDisplayType } from '../../../core/type';

export enum ActionType {
  SetHosts = '[app.target] SetHosts',
  SetPorts = '[app.target] SetPorts',
  SetServices = '[app.target] SetServices',

  ChangeTargetDisplayType = '[app.target] ChangeTargetDisplayType',
  RefreshTargetDisplay = '[app.target] RefreshTargetDisplay',
  ChangeSelectedTarget = '[app.target] ChangeSelectedTarget',
  ChangeTargetDisplayElementRef = '[app.target] ChangeTargetDisplayElementRef',
}

export class SetHosts implements Action {
  readonly type = ActionType.SetHosts;

  constructor(public payload: Host[]) { }
}

export class SetPorts implements Action {
  readonly type = ActionType.SetPorts;

  constructor(public payload: Port[]) { }
}

export class SetServices implements Action {
  readonly type = ActionType.SetServices;

  constructor(public payload: Service[]) { }
}

export class ChangeTargetDisplayType implements Action {
  readonly type = ActionType.ChangeTargetDisplayType;

  constructor(public payload: { targetDisplayType: TargetDisplayType }) { }
}

export class RefreshTargetDisplay implements Action {
  readonly type = ActionType.RefreshTargetDisplay;

  constructor(public payload: boolean) { }
}

export class ChangeSelectedTarget implements Action {
  readonly type = ActionType.ChangeSelectedTarget;

  constructor(public payload: { group: string, target: Host | Port | Service }) { }
}

export class ChangeTargetDisplayElementRef implements Action {
  readonly type = ActionType.ChangeTargetDisplayElementRef;

  constructor(public payload: { targetDisplayElementRef: ElementRef }) { }
}

export type Actions =
  | SetHosts
  | SetPorts
  | SetServices
  | ChangeTargetDisplayType
  | RefreshTargetDisplay
  | ChangeSelectedTarget
  | ChangeTargetDisplayElementRef

  ;
